@@ -165,8 +165,9 @@ computer_clock_update_func (ClockData * cd,
 	ComputerClock *cc;
 	struct tm *tm;
 	GString *gs = g_string_new("");
-	char date[256], hour[256], tooltip[256];
-
+	char date[256], hour[256];
+	char *utf8;
+	
 	cc = g_object_get_data (G_OBJECT (cd->clockw), "cc");
 
 	if (cd->gmt_time)
@@ -177,7 +178,7 @@ computer_clock_update_func (ClockData * cd,
 	if (cd->unixtime) {
 		if ((cd->orient == PANEL_APPLET_ORIENT_LEFT ||
 		     cd->orient == PANEL_APPLET_ORIENT_RIGHT) &&
-		    cd->size >= GNOME_PANEL_MEDIUM) {
+		    cd->size >= GNOME_Vertigo_PANEL_MEDIUM) {
 			g_snprintf (hour, sizeof(hour), "%lu\n%05lu",
 				    (unsigned long)(current_time / 100000L),
 				    (unsigned long)(current_time % 100000L));
@@ -194,7 +195,7 @@ computer_clock_update_func (ClockData * cd,
 		   12 hour format.  */
 		if ((cd->orient == PANEL_APPLET_ORIENT_LEFT ||
 		     cd->orient == PANEL_APPLET_ORIENT_RIGHT) &&
-		    cd->size >= GNOME_PANEL_MEDIUM) {
+		    cd->size >= GNOME_Vertigo_PANEL_MEDIUM) {
 			if (strftime (hour, sizeof (hour), _("%I:%M\n%p"), tm) <= 0)
 				strcpy (hour, "???");
 		} else {
@@ -213,7 +214,7 @@ computer_clock_update_func (ClockData * cd,
 	if (cd->showdate && !cd->unixtime && !cd->internettime) {
 		if ((cd->orient == PANEL_APPLET_ORIENT_LEFT ||
 		     cd->orient == PANEL_APPLET_ORIENT_RIGHT) &&
-		    cd->size >= GNOME_PANEL_MEDIUM) {
+		    cd->size >= GNOME_Vertigo_PANEL_MEDIUM) {
 			/* This format string is used, to display the actual day,
 			   when showing a vertical panel.  For an explanation of
 			   this format string type 'man strftime'.  */
@@ -225,7 +226,7 @@ computer_clock_update_func (ClockData * cd,
 			if (strftime (date, sizeof (date), _("%a %b %d"), tm) <= 0)
 				strcpy (date, "???");
 		}
-		if (cd->size < GNOME_PANEL_MEDIUM)
+		if (cd->size < GNOME_Vertigo_PANEL_MEDIUM)
 			g_string_append_c (gs,' ');
 		else
 			g_string_append_c (gs,'\n');
@@ -244,7 +245,7 @@ computer_clock_update_func (ClockData * cd,
 	/*if we are vertical, just make it char per line */
 	if ((cd->orient == PANEL_APPLET_ORIENT_LEFT ||
 	     cd->orient == PANEL_APPLET_ORIENT_RIGHT) &&
-	    cd->size < GNOME_PANEL_MEDIUM) {
+	    cd->size < GNOME_Vertigo_PANEL_MEDIUM) {
 		char *p;
 		GString *gst = g_string_new ("");
 		
@@ -256,8 +257,12 @@ computer_clock_update_func (ClockData * cd,
 		g_string_free (gs, TRUE);
 		gs = gst;
 	}
-	gtk_label_set_text (GTK_LABEL (cc->time), gs->str);
+
+	utf8 = g_locale_to_utf8 (gs->str, -1, NULL, NULL, NULL);
+
+	gtk_label_set_text (GTK_LABEL (cc->time), utf8);
 	g_string_free (gs, TRUE);
+	g_free (utf8);
 }
 
 static void
@@ -283,6 +288,7 @@ create_computer_clock_widget (GtkWidget ** clock, ClockUpdateFunc * update_func)
 	gtk_widget_show (vbox);
 
 	cc->time = gtk_label_new ("hmm?");
+	gtk_label_set_justify (GTK_LABEL (cc->time), GTK_JUSTIFY_CENTER);
 
 	gtk_box_pack_start_defaults (GTK_BOX (vbox), cc->time);
 	gtk_widget_show (cc->time);
@@ -325,8 +331,8 @@ create_clock_widget (ClockData *cd)
 
 	cd->props = NULL;
 
-	cd->orient = PANEL_APPLET_ORIENT_UP;
-	cd->size = GNOME_PANEL_MEDIUM;
+	cd->orient = panel_applet_get_orient (PANEL_APPLET (cd->applet));
+	cd->size = panel_applet_get_size (PANEL_APPLET (cd->applet));
 
 	gtk_signal_connect (GTK_OBJECT(clock), "destroy",
 			    (GtkSignalFunc) destroy_clock,
@@ -368,6 +374,29 @@ applet_change_orient (PanelApplet       *applet,
 	(*cd->update_func) (cd, current_time);
 }
 
+static void
+applet_change_background (PanelApplet               *applet,
+			  PanelAppletBackgroundType  type,
+			  GdkColor                  *color,
+			  const gchar               *pixmap,
+			  ClockData                 *cd)
+{
+  if (type == PANEL_NO_BACKGROUND)
+    {
+      GtkRcStyle *rc_style = gtk_rc_style_new ();
+
+      gtk_widget_modify_style (cd->applet, rc_style);
+    }
+  else if (type == PANEL_COLOR_BACKGROUND)
+    {
+      gtk_widget_modify_bg (cd->applet,
+			    GTK_STATE_NORMAL,
+			    color);
+    } else { /* pixmap */
+      /* FIXME: Handle this when the panel support works again */
+    }
+}
+
 
 /* this is when the panel size changes */
 static void
@@ -381,7 +410,7 @@ applet_change_pixel_size (PanelApplet *applet,
 	/* Adjust the padding on the text for small sizes of the panel (so the widget doesn't become bigger than the panel */
 	cc = g_object_get_data (G_OBJECT (cd->clockw), "cc");
 	gtk_container_set_border_width (GTK_CONTAINER (cc->align),
-					((size < GNOME_PANEL_SMALL)? 0 : 3 ) );
+					((size < GNOME_Vertigo_PANEL_SMALL)? 0 : 3 ) );
 	
 	time (&current_time);
 	cd->size = size;
@@ -492,13 +521,15 @@ static const char clock_menu_xml [] =
 	"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
 	"</popup>\n";
 
-BonoboObject *
-make_clock_applet(void)
+gboolean
+fill_clock_applet(PanelApplet *applet)
 {
 	ClockData *cd;
 	
 	cd = g_new0 (ClockData, 1);
 
+	cd->applet = GTK_WIDGET (applet);
+
 	/* FIXME: The config stuff needs to be completely redone... */
 	
 	//	gnome_config_push_prefix(APPLET_WIDGET(applet)->privcfgpath);
@@ -552,7 +583,7 @@ make_clock_applet(void)
 
 	create_clock_widget (cd);
 
-	cd->applet = panel_applet_new (cd->clockw);
+	gtk_container_add (GTK_CONTAINER (cd->applet), cd->clockw);
 
 	gtk_widget_show (cd->applet);
 
@@ -571,6 +602,13 @@ make_clock_applet(void)
 			  G_CALLBACK (applet_change_pixel_size),
 			  cd);
 
+	/* FIXME: initial background, this needs some panel-applet voodoo */
+	g_signal_connect (G_OBJECT (cd->applet),
+			  "change_background",
+			  G_CALLBACK (applet_change_background),
+			  cd);
+
+	
 #ifdef FIXME
 	g_signal_connect (G_OBJECT (cd->applet),
 			  "save_session",
@@ -580,7 +618,7 @@ make_clock_applet(void)
 
 	panel_applet_setup_menu (PANEL_APPLET (cd->applet), clock_menu_xml, clock_menu_verbs, cd);
 	
-	return BONOBO_OBJECT (panel_applet_get_control (PANEL_APPLET (cd->applet)));
+	return TRUE;
 }
 
 static void
