@@ -76,9 +76,10 @@ panel_applet_clean_gconf (AppletInfo *info)
 	GConfClient *client;
 	GSList      *id_list, *l;
 	char        *temp_key = NULL;
-	char        *profile;
+	const char  *profile;
 
-	g_return_if_fail (info && info->gconf_key);
+	g_return_if_fail (info != NULL);
+	g_return_if_fail (info->gconf_key != NULL);
 
         client  = panel_gconf_get_client ();
 	profile = session_get_current_profile ();
@@ -118,6 +119,9 @@ panel_applet_clean_gconf (AppletInfo *info)
 
         g_free (temp_key);
 	panel_g_slist_deep_free (id_list);
+
+	g_free (info->gconf_key);
+	info->gconf_key = NULL;
 }
 
 /*destroy widgets and call the above cleanup function*/
@@ -131,22 +135,22 @@ panel_applet_clean (AppletInfo *info)
 		info->remove_idle = 0;
 	}
 
-	panel_applet_clean_gconf (info);
-
 	applets = g_slist_remove (applets, info);
 
-	if (info->widget != NULL) {
+	if (info->widget) {
 		GtkWidget *widget = info->widget;
-		if(info->type == APPLET_STATUS) {
+
+		if (info->type == APPLET_STATUS)
 			status_applet_put_offscreen (info->data);
-		}
-		/* destroy will remove it from the panel */
+
 		info->widget = NULL;
 		gtk_widget_destroy (widget);
 	}
 
 	info->data = NULL;
 
+	panel_applet_clean_gconf (info);
+
 	g_free (info);
 }
 
@@ -786,16 +790,11 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 		info->menu_age = 0;
 	}
 
-	info->type = APPLET_EMPTY;
-
 	if (info->data_destroy)
 		info->data_destroy (info->data);
 	info->data_destroy = NULL;
 	info->data = NULL;
 
-	g_free (info->gconf_key);
-	info->gconf_key = NULL;
-
 	/*free the user menu*/
 	for(li = info->user_menu; li != NULL; li = g_list_next(li)) {
 		AppletUserMenu *umenu = li->data;
@@ -839,6 +838,37 @@ panel_applet_get_full_gconf_key (AppletType   type,
 	return retval;
 }
 
+typedef struct {
+	PanelWidget *panel_widget;
+	int          position;
+	char        *unique_id;
+	gboolean     use_default;
+} PanelAppletToLoad;
+
+static GSList *panel_applets_to_load = NULL;
+
+static gboolean
+panel_applet_load_idle_handler (gpointer dummmy)
+{
+	PanelAppletToLoad *applet;
+
+	if (!panel_applets_to_load)
+		return FALSE;
+
+	applet = (PanelAppletToLoad *) panel_applets_to_load->data;
+	panel_applets_to_load =
+		g_slist_remove (panel_applets_to_load, applet);
+
+	panel_applet_frame_load_from_gconf (
+			applet->panel_widget, applet->position,
+			applet->unique_id, applet->use_default);
+
+	g_free (applet->unique_id);
+	g_free (applet);
+
+	return TRUE;
+}
+
 static void
 panel_applet_load_from_unique_id (AppletType   type,
 				  GConfClient *gconf_client,
@@ -887,8 +917,21 @@ panel_applet_load_from_unique_id (AppletType   type,
 	position += right_stick ? G_MAXINT/2 : 0;
 
 	switch (applet_type) {
-	case APPLET_BONOBO:
-		panel_applet_frame_load_from_gconf (panel_widget, position, unique_id, use_default);
+	case APPLET_BONOBO: {
+		PanelAppletToLoad *applet;
+
+		applet = g_new0 (PanelAppletToLoad, 1);
+
+		applet->panel_widget = panel_widget;
+		applet->position     = position;
+		applet->unique_id    = g_strdup (unique_id);
+		applet->use_default  = use_default;
+
+		if (!panel_applets_to_load)
+			g_idle_add (panel_applet_load_idle_handler, NULL);
+
+		panel_applets_to_load = g_slist_prepend (panel_applets_to_load, applet);
+		}
 		break;
 	case APPLET_DRAWER:
 		drawer_load_from_gconf (panel_widget, position, unique_id, use_default);
@@ -922,7 +965,7 @@ panel_applet_load_list (AppletType   type,
 {
 	GConfClient *client;
 	GSList      *id_list, *l;
-	char        *profile;
+	const char  *profile;
 	char        *temp_key = NULL;
 	
 	client = panel_gconf_get_client ();
@@ -970,7 +1013,7 @@ panel_applet_save_position (AppletInfo *applet_info,
 			    const char *gconf_key)
 {
 	GConfClient *client;
-	char        *profile;
+	const char  *profile;
 	char        *temp_key;
 
 	client  = panel_gconf_get_client ();
@@ -995,7 +1038,7 @@ void
 panel_applet_save_to_gconf (AppletInfo *applet_info)
 {
 	GConfClient *client;
-	char        *profile;
+	const char  *profile;
 	char        *temp_key;
 	GSList      *id_list, *l;
 
