@@ -59,6 +59,12 @@ enum {
 	NUM_COLUMNS
 };
 
+typedef enum {
+	PANEL_RESPONSE_RUN,
+} PanelResponseType;
+
+#define PANEL_STOCK_RUN "panel-run"
+
 extern GtkTooltips *panel_tooltips;
 extern gboolean no_run_box;
 
@@ -100,8 +106,7 @@ fill_executables (void)
 	const char *path;
 	char **pathv;
 
-	g_list_foreach (executables, (GFunc) g_free, NULL);
-	g_list_free (executables);
+	panel_g_list_deep_free (executables);
 	executables = NULL;
 
 	path = g_getenv ("PATH");
@@ -133,8 +138,7 @@ static void
 kill_completion (void)
 {
 	if (executables != NULL) {
-		g_list_foreach (executables, (GFunc) g_free, NULL);
-		g_list_free (executables);
+		panel_g_list_deep_free (executables);
 		executables = NULL;
 	}
 
@@ -163,8 +167,7 @@ get_environment (int *argc, char ***argv, int *envc, char ***envv)
 	}
 
 	if (moveby == *argc) {
-		g_list_foreach (envar, (GFunc) g_free, NULL);
-		g_list_free (envar);
+		panel_g_list_deep_free (envar);
 		return;
 	}
 
@@ -192,10 +195,6 @@ get_environment (int *argc, char ***argv, int *envc, char ***envv)
 	g_list_free (envar);
 }
 
-enum {
-	RUN_BUTTON
-};
-
 static void 
 run_dialog_response (GtkWidget *w, int response, gpointer data)
 {
@@ -310,6 +309,9 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 		} else if (strcmp (s, "gegls from outer space") == 0) {
 			start_geginv ();
                         goto return_and_close;
+		} else if (strcmp (s, "End world hunger") == 0) {
+			gnome_url_show ("http://www.wfp.org", NULL);
+                        goto return_and_close;
 		}
 
                 /* Somewhat of a hack I suppose */
@@ -620,7 +622,7 @@ advanced_contents_shown (GtkWidget *vbox,
 static void
 activate_run (GtkWidget *entry, GtkWidget *dialog)
 {
-	gtk_dialog_response (GTK_DIALOG (dialog), RUN_BUTTON);
+	gtk_dialog_response (GTK_DIALOG (dialog), PANEL_RESPONSE_RUN);
 }
 
 static GtkWidget*
@@ -652,7 +654,6 @@ create_advanced_contents (void)
 			  G_CALLBACK (kill_completion),
 			  NULL);
  
-        gtk_window_set_focus (GTK_WINDOW (run_dialog), entry);
         gtk_combo_set_use_arrows_always (GTK_COMBO (gentry), TRUE);
         g_object_set_data (G_OBJECT (run_dialog), "entry", entry);
 
@@ -678,11 +679,14 @@ create_advanced_contents (void)
         gtk_box_pack_start (GTK_BOX (vbox), w,
                             FALSE, FALSE, GNOME_PAD_SMALL);
         
-        g_object_ref (G_OBJECT (vbox));
-        
         g_object_set_data_full (G_OBJECT (run_dialog),
 				"advanced",
-				vbox,
+				g_object_ref (vbox),
+				(GDestroyNotify) g_object_unref);
+
+        g_object_set_data_full (G_OBJECT (run_dialog),
+				"advanced-entry",
+				g_object_ref (entry),
 				(GDestroyNotify) g_object_unref);
         
         g_signal_connect (G_OBJECT (vbox),
@@ -710,24 +714,20 @@ add_columns (GtkTreeView *treeview)
 	GtkTreeViewColumn *column;
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
-	column = gtk_tree_view_column_new_with_attributes (NULL,
-							   renderer,
-							   "pixbuf", COLUMN_ICON,
-							   NULL);
-	gtk_tree_view_column_set_clickable (column, FALSE);
-	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
-					 GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column),
-					      ICON_SIZE);
-	gtk_tree_view_append_column (treeview, column);
-
+	column = gtk_tree_view_column_new ();
+        gtk_tree_view_column_set_title (column, _("Applications"));
+                
+        gtk_tree_view_column_pack_start (column, renderer, FALSE);
+        gtk_tree_view_column_set_attributes (column, renderer,
+                                             "pixbuf", COLUMN_ICON,
+                                             NULL);
+        
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Applications"),
-							   renderer,
-							   "text",
-							   COLUMN_FULLNAME,
-							   NULL);
-	gtk_tree_view_column_set_clickable (column, FALSE);
+        gtk_tree_view_column_pack_start (column, renderer, TRUE);
+
+        gtk_tree_view_column_set_attributes (column, renderer,
+                                             "text", COLUMN_FULLNAME,
+                                             NULL);
 	gtk_tree_view_append_column (treeview, column);
 }
 
@@ -833,7 +833,7 @@ fill_list (GtkWidget *list)
 	add_columns (GTK_TREE_VIEW (list));
 }
 
-#define DEFAULT_ICON "nautilus/i-executable.png"
+#define DEFAULT_ICON "document-icons/i-executable.png"
 #define FALLBACK_DEFAULT_ICON "gnome-logo-icon-transparent.png"
 
 static void
@@ -1006,6 +1006,8 @@ create_simple_contents (void)
 			  run_dialog);
         
         w = gtk_scrolled_window_new (NULL, NULL);
+        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (w),
+                                             GTK_SHADOW_IN);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (w),
                                         GTK_POLICY_AUTOMATIC,
                                         GTK_POLICY_AUTOMATIC);
@@ -1077,13 +1079,18 @@ update_contents (GtkWidget *dialog)
                 advanced = g_object_get_data (G_OBJECT (dialog), "advanced");
                 
                 if (advanced && advanced->parent == NULL) {
+			GtkWidget *advanced_entry;
+
+			advanced_entry =
+				g_object_get_data (G_OBJECT (dialog), "advanced-entry");
+
                         gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
                                             advanced,
                                             FALSE, FALSE, 0);
                 
                         gtk_widget_show_all (advanced);
 
-                        gtk_widget_grab_focus (advanced);
+                        gtk_widget_grab_focus (advanced_entry);
                 }
 
                 gtk_label_set_text (GTK_LABEL (advanced_toggle),
@@ -1109,19 +1116,49 @@ update_contents (GtkWidget *dialog)
         }
 }
 
+static inline void
+register_run_stock_item (void)
+{
+	static gboolean registered = FALSE;
+
+	if (!registered) {
+		GtkIconFactory      *factory;
+		GtkIconSet          *execute_icons;
+
+		static GtkStockItem  run_item [] = {
+			{ PANEL_STOCK_RUN, N_("_Run ..."), 0, 0, GETTEXT_PACKAGE },
+		};
+
+		execute_icons = gtk_icon_factory_lookup_default (GTK_STOCK_EXECUTE);
+
+		factory = gtk_icon_factory_new ();
+
+		gtk_icon_factory_add (factory, PANEL_STOCK_RUN, execute_icons);
+
+		gtk_icon_factory_add_default (factory);
+
+		gtk_stock_add_static (run_item, 1);
+
+		registered = TRUE;
+	}
+}
+
 void
 show_run_dialog (void)
 {
-        gboolean use_advanced;          
+        gboolean  use_advanced;          
+	char     *run_icon;
 
 	if (no_run_box)
 		return;
 
-	if(run_dialog != NULL) {
+	if (run_dialog != NULL) {
 		gtk_window_present (GTK_WINDOW (run_dialog));
 		return;
 	}
 
+	register_run_stock_item ();
+
         use_advanced = gnome_config_get_bool ("/panel/State/"ADVANCED_DIALOG_KEY"=false");
         
 	run_dialog = gtk_dialog_new_with_buttons (_("Run Program"),
@@ -1131,10 +1168,8 @@ show_run_dialog (void)
 						  GTK_RESPONSE_HELP,
 						  GTK_STOCK_CLOSE,
 						  GTK_RESPONSE_CLOSE,
-						  /* FIXME: how the hell do we get a different label but
-						   * the execute stock icon */
-						  GTK_STOCK_EXECUTE,
-						  RUN_BUTTON,
+						  PANEL_STOCK_RUN,
+						  PANEL_RESPONSE_RUN,
 						  NULL);
 
         /* This is lame in advanced mode, but if you change it on mode
@@ -1150,19 +1185,21 @@ show_run_dialog (void)
         if (!use_advanced)
                 gtk_window_set_default_size (GTK_WINDOW (run_dialog),
                                              -1, 400);
-        
-#ifdef FIXME
-	gnome_window_icon_set_from_file (GTK_WINDOW (run_dialog),
-					 GNOME_ICONDIR"/gnome-run.png");
-#endif
+
+	run_icon = gnome_program_locate_file (
+			NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-run.png", TRUE, NULL);
+	if (run_icon) {
+		gnome_window_icon_set_from_file (GTK_WINDOW (run_dialog), run_icon);
+		g_free (run_icon);
+	}
+
 	g_signal_connect(G_OBJECT(run_dialog), "destroy",
 			   G_CALLBACK(gtk_widget_destroyed),
 			   &run_dialog);
 	gtk_window_set_position (GTK_WINDOW (run_dialog), GTK_WIN_POS_MOUSE);
 	gtk_window_set_wmclass (GTK_WINDOW (run_dialog), "run_dialog", "Panel");
 
-	gtk_dialog_set_default_response (GTK_DIALOG (run_dialog), 
-					 RUN_BUTTON);
+	gtk_dialog_set_default_response (GTK_DIALOG (run_dialog), PANEL_RESPONSE_RUN);
 
         g_signal_connect (G_OBJECT (run_dialog), "response", 
 			  G_CALLBACK (run_dialog_response), NULL);
