@@ -17,29 +17,24 @@
 #include <string.h>
 #include <limits.h>
 #include <errno.h>
-#include <gnome.h>
 
-#include "panel-include.h"
+#include <libgnomeui/libgnomeui.h>
+#include <libgnome/libgnome.h>
 
-#include "icon-entry-hack.h"
+#include "gnome-desktop-item.h"
 
-/*#define PANEL_DEBUG 1*/
+#include "quick-desktop-reader.h"
+#include "panel-util.h"
 
-extern GlobalConfig global_config;
-
-extern int config_sync_timeout;
-extern int panels_to_sync;
-extern int applets_to_sync;
-extern int need_complete_save;
+#include "menu-properties.h"
 
+#undef MENU_PROPERTIES_DEBUG
 
 struct _MenuDialogInfo {
 	GtkWidget *main_menu;
 	GtkWidget *global_main;
 	GtkWidget *system;
 	GtkWidget *system_sub;
-	GtkWidget *user;
-	GtkWidget *user_sub;
 	GtkWidget *applets;
 	GtkWidget *applets_sub;
 	GtkWidget *distribution;
@@ -59,85 +54,55 @@ struct _MenuDialogInfo {
 };
 
 char *
-get_real_menu_path(const char *arguments)
+get_real_menu_path (const char *arguments, gboolean main_menu)
 {
-	char *this_menu;
+	if (main_menu)
+		return g_strdup ("applications:");
 	
-	/*if null, let's put the main menu up*/
-	if (string_empty (arguments))
-		arguments = ".";
-
-	if(strcmp(arguments, ".") == 0)
-		this_menu = gnome_unconditional_datadir_file ("gnome/apps");
-	else if (*arguments == '/')
-		this_menu = g_strdup (arguments);
 	else if (*arguments == '~')
-		this_menu = g_concat_dir_and_file (g_get_home_dir(),
-						   &arguments[1]);
+		/* FIXME: this needs to be a URI */
+		return g_build_filename (g_get_home_dir(),
+					      &arguments[1], NULL);
 	else
-		this_menu = gnome_unconditional_datadir_file (arguments);
-
-	if (!panel_file_exists (this_menu)) {
-		g_warning("menu %s does not exist "
-			  "(arguments are %s)",
-			  this_menu, arguments);
-		g_free (this_menu);
-		return NULL;
-	}
-	return this_menu;
+		return g_strdup (arguments);
 }
 
 char *
-get_pixmap(const char *menudir, gboolean main_menu)
+get_pixmap (const char *menudir, gboolean main_menu)
 {
 	char *pixmap_name = NULL;
+
 	if (main_menu) {
-		pixmap_name = gnome_unconditional_pixmap_file("gnome-logo-icon-transparent.png");
+		pixmap_name = panel_pixmap_discovery ("gnome-logo-icon-transparent.png",
+						      TRUE /* fallback */);
 	} else {
 		char *dentry_name;
-		GnomeDesktopEntry *item_info;
+		QuickDesktopItem *qitem;
 
-		dentry_name = g_concat_dir_and_file (menudir,
-						     ".directory");
-		item_info = gnome_desktop_entry_load (dentry_name);
+		dentry_name = g_build_path ("/",
+					    menudir,
+					    ".directory",
+					    NULL);
+		qitem = quick_desktop_item_load_uri (dentry_name,
+						     NULL /* expected_type */,
+						     FALSE /* run_tryexec */);
 		g_free (dentry_name);
 
-		if(item_info && item_info->icon)
-			pixmap_name = g_strdup(item_info->icon);
-		else
-			pixmap_name =
-				gnome_unconditional_pixmap_file ("gnome-folder.png");
+		if (qitem != NULL)
+			pixmap_name = gnome_desktop_item_find_icon (qitem->icon,
+								    20 /* desired size */,
+								    0 /* flags */);
 
-		if (item_info)
-			gnome_desktop_entry_free(item_info);
+		if (pixmap_name == NULL)
+			pixmap_name = panel_pixmap_discovery ("gnome-folder.png",
+							      TRUE /* fallback */);
+
+		if (qitem != NULL)
+			quick_desktop_item_destroy (qitem);
 	}
 	return pixmap_name;
 }
 
-/* Cache this, we don't want to check in the path all the time */
-static gboolean
-got_gmenu (void)
-{
-	static gboolean checked = FALSE;
-	static gboolean got_it = FALSE;
-	char *tmp;
-
-	if (checked)
-		return got_it;
-
-	tmp = panel_is_program_in_path ("gmenu");
-	if (tmp != NULL)
-		got_it = TRUE;
-	else
-		got_it = FALSE;
-
-	g_free (tmp);
-
-	checked = TRUE;
-
-	return got_it;
-}
-
 static void
 properties_apply_callback (Menu *menu)
 {
@@ -164,7 +129,7 @@ properties_apply_callback (Menu *menu)
 		change_icon = TRUE;
 	}
 
-	s = hack_icon_entry_get_icon (GNOME_ICON_ENTRY(menu->dialog_info->custom_icon_entry));
+	s = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY(menu->dialog_info->custom_icon_entry));
 	if (menu->custom_icon_file == NULL ||
 	    s == NULL ||
 	    strcmp (menu->custom_icon_file, s) != 0) {
@@ -176,41 +141,43 @@ properties_apply_callback (Menu *menu)
 	}
 
 	need_edit_menus = FALSE;
+
+	/* default to non-main-menu */
+	menu->main_menu = FALSE;
+
 	if (GTK_TOGGLE_BUTTON (menu->dialog_info->main_menu)->active ||
 	    GTK_TOGGLE_BUTTON (menu->dialog_info->global_main)->active) {
-		g_free (menu->path);
-		menu->path = g_strdup (".");
+		menu->main_menu = TRUE;
 
-		if (got_gmenu ()) {
-			need_edit_menus = TRUE;
-		}
+		need_edit_menus = TRUE;
 	} else {
-		g_free (menu->path);
 		s = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (menu->dialog_info->pathentry),
 						    TRUE);
 		if(s == NULL) {
 			g_warning (_("Can't open directory, using main menu!"));
-			menu->path = g_strdup (".");
+			menu->main_menu = TRUE;
 		} else if (*s == '\0') {
-			menu->path = g_strdup (".");
+			menu->main_menu = TRUE;
 		} else {
+			g_free (menu->path);
 			menu->path = g_strdup (s);
 		}
 	}
 
-
 	/* Setup the edit_menus callback */
-	if (applet_get_callback (menu->info->user_menu, "edit_menus") != NULL)
+	if (panel_applet_get_callback (menu->info->user_menu, "edit_menus"))
 		got_edit_menus = TRUE;
 	else
 		got_edit_menus = FALSE;
 
 	if (need_edit_menus && ! got_edit_menus)
-		applet_add_callback (menu->info, "edit_menus",
-				     NULL,
-				     _("Edit menus..."));
-	else if ( ! need_edit_menus && got_edit_menus)
-		applet_remove_callback (menu->info, "edit_menus");
+		panel_applet_add_callback (menu->info,
+					   "edit_menus",
+					   NULL,
+					   _("Edit menus..."));
+
+	else if (!need_edit_menus && got_edit_menus)
+		panel_applet_remove_callback (menu->info, "edit_menus");
 
 
 
@@ -225,11 +192,6 @@ properties_apply_callback (Menu *menu)
 	else if (GTK_TOGGLE_BUTTON(menu->dialog_info->system)->active)
 		menu->main_menu_flags |= MAIN_MENU_SYSTEM;
 
-	if(GTK_TOGGLE_BUTTON(menu->dialog_info->user_sub)->active)
-		menu->main_menu_flags |= MAIN_MENU_USER_SUB;
-	else if (GTK_TOGGLE_BUTTON (menu->dialog_info->user)->active)
-		menu->main_menu_flags |= MAIN_MENU_USER;
-
 	if(GTK_TOGGLE_BUTTON(menu->dialog_info->applets_sub)->active)
 		menu->main_menu_flags |= MAIN_MENU_APPLETS_SUB;
 	else if (GTK_TOGGLE_BUTTON (menu->dialog_info->applets)->active)
@@ -271,12 +233,11 @@ properties_apply_callback (Menu *menu)
 
 	/* Apply menu changes */
 	if (need_reload) {
-		char *this_menu = get_real_menu_path (menu->path);
+		char *this_menu = get_real_menu_path (menu->path, menu->main_menu);
 		GSList *list = g_slist_append (NULL, this_menu);
-		gboolean main_menu = strcmp (menu->path, ".") == 0;
 		
 		add_menu_widget (menu, PANEL_WIDGET (menu->button->parent),
-				 list, main_menu, TRUE);
+				 list, TRUE);
 		
 		g_free (this_menu);
 
@@ -285,18 +246,21 @@ properties_apply_callback (Menu *menu)
 
 	/* Apply icon changes */
 	if (change_icon) {
-		char *this_menu = get_real_menu_path(menu->path);
+		char *this_menu = get_real_menu_path (menu->path,
+						      menu->main_menu);
 		char *pixmap_name;
 
 		if (menu->custom_icon &&
 		    menu->custom_icon_file != NULL &&
-		    panel_file_exists (menu->custom_icon_file))
+		    g_file_test (menu->custom_icon_file, G_FILE_TEST_EXISTS))
 			pixmap_name = g_strdup (menu->custom_icon_file);
 		else
-			pixmap_name = get_pixmap(this_menu, (strcmp (menu->path, ".") == 0));
+			pixmap_name = get_pixmap (this_menu, menu->main_menu);
 		button_widget_set_pixmap(BUTTON_WIDGET(menu->button),
 					 pixmap_name, -1);
 	}
+
+	panel_applet_save_to_gconf (menu->info);
 }
 
 static void
@@ -320,11 +284,11 @@ toggle_prop(GtkWidget *widget, gpointer data)
 }
 
 static void
-toggle_global_main(GtkWidget *widget, gpointer data)
+toggle_global_main (GtkWidget *widget, gpointer data)
 {
 	Menu *menu = data;
 
-	if(GTK_TOGGLE_BUTTON(widget)->active) {
+	if (GTK_TOGGLE_BUTTON (widget)->active) {
 		gtk_widget_set_sensitive(menu->dialog_info->main_frame, FALSE);
 		gtk_widget_set_sensitive(menu->dialog_info->normal_frame, FALSE);
 
@@ -396,50 +360,50 @@ add_menu_type_options(Menu *menu, GtkObject *dialog, GtkTable *table, int row,
 
 	if(!on && !sub)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
-			    GTK_SIGNAL_FUNC (toggle_prop), 
-			    menu);
+	g_signal_connect (G_OBJECT (w), "toggled", 
+			  G_CALLBACK (toggle_prop), 
+			  menu);
 	
-	w = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(rb)),
+	w = gtk_radio_button_new_with_label (gtk_radio_button_get_group (GTK_RADIO_BUTTON (rb)),
 					     _("In a submenu"));
-	gtk_table_attach_defaults(table, w, 2, 3, row, row+1);
+	gtk_table_attach_defaults (table, w, 2, 3, row, row+1);
 
 	*widget_sub = w;
 
 	if (sub)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
-			    GTK_SIGNAL_FUNC (toggle_prop), 
-			    menu);
+	g_signal_connect (G_OBJECT (w), "toggled", 
+			  G_CALLBACK (toggle_prop), 
+			  menu);
 	
-	w = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(rb)),
+	w = gtk_radio_button_new_with_label (gtk_radio_button_get_group (GTK_RADIO_BUTTON (rb)),
 					     _("On the main menu"));
 	*widget = w;
 	gtk_table_attach_defaults(table, w, 1, 2, row, row+1);
 	if (on)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
-			    GTK_SIGNAL_FUNC (toggle_prop), 
-			    menu);
+	g_signal_connect (G_OBJECT (w), "toggled", 
+			  G_CALLBACK (toggle_prop), 
+			  menu);
 }
 
 static void
-dialog_clicked (GtkWidget *widget, int button, gpointer data)
+dialog_response (GtkWidget *dialog, int response, gpointer data)
 {
 	Menu *menu = data;
 
-	if (button == 0 /* close */) {
-		gnome_dialog_close (GNOME_DIALOG (widget));
-	} else if (button == 1 /* help */) {
+	if (response == GTK_RESPONSE_CLOSE) {
+		gtk_widget_destroy (dialog);
+	} else if (response == GTK_RESPONSE_HELP) {
 		if (GTK_TOGGLE_BUTTON (menu->dialog_info->main_menu)->active)
-			panel_show_help ("mainmenu.html#MAINMENUCONFIG");
+			panel_show_help ("mainmenu", "MAINMENUCONFIG");
 		else
-			panel_show_help ("menus.html");
+			panel_show_help ("menus", NULL);
 	}
 }
 
 static GtkWidget *
-create_properties_dialog(Menu *menu)
+create_properties_dialog (Menu *menu)
 {
 	GtkWidget *dialog, *notebook;
 	GtkWidget *vbox;
@@ -450,26 +414,25 @@ create_properties_dialog(Menu *menu)
 	GtkWidget *t;
 	GtkWidget *main_menu, *global_main;
 
-	dialog = gnome_dialog_new (_("Menu properties"),
-				   GNOME_STOCK_BUTTON_CLOSE,
-				   GNOME_STOCK_BUTTON_HELP,
-				   NULL);
-	gnome_dialog_set_close (GNOME_DIALOG (dialog),
-				FALSE /* click_closes */);
+	dialog = gtk_dialog_new_with_buttons (_("Menu properties"),
+					      NULL /* parent */,
+					      0 /* flags */,
+					      GTK_STOCK_HELP,
+					      GTK_RESPONSE_HELP,
+					      GTK_STOCK_CLOSE,
+					      GTK_RESPONSE_CLOSE,
+					      NULL);
 
 	menu->prop_dialog = dialog;
 
 	menu->dialog_info = g_new0 (MenuDialogInfo, 1);
 
 	notebook = gtk_notebook_new ();
-	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    notebook, TRUE, TRUE, 0);
 
 	gtk_window_set_wmclass(GTK_WINDOW(dialog),
 			       "menu_properties", "Panel");
-	gtk_window_set_title(GTK_WINDOW(dialog), _("Menu properties"));
-	/*gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);*/
-	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
 	
 	vbox = gtk_vbox_new(FALSE,GNOME_PAD_SMALL);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox),GNOME_PAD_SMALL);
@@ -484,43 +447,43 @@ create_properties_dialog(Menu *menu)
 	w = gtk_radio_button_new_with_label (NULL, _("Global main menu"));
 	global_main = w;
 	menu->dialog_info->global_main = w;
-	if((menu->path == NULL ||
-	    strcmp(menu->path, ".") == 0) &&
-	   menu->global_main)
+	if (menu->main_menu &&
+	    menu->global_main)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
-			    GTK_SIGNAL_FUNC (toggle_global_main), 
-			    menu);
+
+	g_signal_connect (G_OBJECT (w), "toggled", 
+			  G_CALLBACK (toggle_global_main), 
+			  menu);
 	gtk_box_pack_start(GTK_BOX(box), w, TRUE, TRUE, 0);
 
 	w = gtk_radio_button_new_with_label (
-		  gtk_radio_button_group (GTK_RADIO_BUTTON (global_main)),
+		  gtk_radio_button_get_group (GTK_RADIO_BUTTON (global_main)),
 		  _("Main menu"));
 	main_menu = w;
 	menu->dialog_info->main_menu = w;
-	if((menu->path == NULL ||
-	    strcmp(menu->path, ".") == 0) &&
-	   ! menu->global_main)
+	if (menu->main_menu &&
+	    ! menu->global_main)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
-			    GTK_SIGNAL_FUNC (toggle_main_menu), 
-			    menu);
+
+	g_signal_connect (G_OBJECT (w), "toggled", 
+			  G_CALLBACK (toggle_main_menu), 
+			  menu);
 	gtk_box_pack_start(GTK_BOX(box), w, TRUE, TRUE, 0);
 
 	w2 = gtk_radio_button_new_with_label (
-		  gtk_radio_button_group (GTK_RADIO_BUTTON (global_main)),
+		  gtk_radio_button_get_group (GTK_RADIO_BUTTON (global_main)),
 		  _("Normal menu"));
-	if(menu->path && strcmp(menu->path, ".") != 0)
+	if ( ! menu->main_menu)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w2), TRUE);
-	gtk_signal_connect (GTK_OBJECT (w2), "toggled", 
-			    GTK_SIGNAL_FUNC (toggle_normal_menu), 
-			    menu);
+
+	g_signal_connect (G_OBJECT (w2), "toggled", 
+			  G_CALLBACK (toggle_normal_menu), 
+			  menu);
 	gtk_box_pack_start(GTK_BOX(box), w2, TRUE, TRUE, 0);
 
 	f = gtk_frame_new(_("Main menu"));
-	if((menu->path &&
-	    strcmp(menu->path, ".") != 0) ||
-	   menu->global_main)
+	if ( ! menu->main_menu ||
+	     menu->global_main)
 		gtk_widget_set_sensitive(f, FALSE);
 	menu->dialog_info->main_frame = f;
 	gtk_box_pack_start(GTK_BOX(vbox), f, FALSE, FALSE, 0);
@@ -536,13 +499,6 @@ create_properties_dialog(Menu *menu)
 			      &menu->dialog_info->system_sub,
 			      menu->main_menu_flags & MAIN_MENU_SYSTEM,
 			      menu->main_menu_flags & MAIN_MENU_SYSTEM_SUB);
-	add_menu_type_options(menu,
-			      GTK_OBJECT(dialog), GTK_TABLE(table),1,
-			      _("Favorites: "),
-			      &menu->dialog_info->user,
-			      &menu->dialog_info->user_sub,
-			      menu->main_menu_flags & MAIN_MENU_USER,
-			      menu->main_menu_flags & MAIN_MENU_USER_SUB);
 	add_menu_type_options(menu,
 			      GTK_OBJECT(dialog), GTK_TABLE(table),2,
 			      _("Applets: "),
@@ -580,11 +536,10 @@ create_properties_dialog(Menu *menu)
 			      menu->main_menu_flags & MAIN_MENU_DESKTOP_SUB);
 
 	f = gtk_frame_new(_("Normal menu"));
-	if(menu->path == NULL ||
-	   strcmp(menu->path, ".") == 0)
-		gtk_widget_set_sensitive(f, FALSE);
+	if ( ! menu->main_menu)
+		gtk_widget_set_sensitive (f, FALSE);
 	menu->dialog_info->normal_frame = f;
-	gtk_box_pack_start(GTK_BOX(vbox), f, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), f, FALSE, FALSE, 0);
 	
 	box = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
 	gtk_container_set_border_width(GTK_CONTAINER(box), GNOME_PAD_SMALL);
@@ -593,20 +548,20 @@ create_properties_dialog(Menu *menu)
 	w = gtk_label_new(_("Menu path"));
 	gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
 
-	w = gnome_file_entry_new("menu_path",_("Browse"));
-	gnome_file_entry_set_directory(GNOME_FILE_ENTRY(w), TRUE);
+	w = gnome_file_entry_new ("menu_path", _("Browse"));
+	gnome_file_entry_set_directory_entry (GNOME_FILE_ENTRY (w), TRUE);
 
 	t = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (w));
 	menu->dialog_info->pathentry = w;
 	if (menu->path) {
-		char *s = get_real_menu_path(menu->path);
+		char *s = get_real_menu_path (menu->path, menu->main_menu);
 		gtk_entry_set_text(GTK_ENTRY(t), s);
 		g_free(s);
 	}
 	gtk_box_pack_start(GTK_BOX(box),w,TRUE,TRUE,0);
-	gtk_signal_connect (GTK_OBJECT (t), "changed",
-			    GTK_SIGNAL_FUNC (textbox_changed),
-			    menu);
+	g_signal_connect (G_OBJECT (t), "changed",
+			  G_CALLBACK (textbox_changed),
+			  menu);
 	
 	gtk_notebook_append_page (GTK_NOTEBOOK(notebook),
 				  vbox, gtk_label_new (_("Menu")));
@@ -626,39 +581,51 @@ create_properties_dialog(Menu *menu)
 	if(menu->custom_icon &&
 	   menu->custom_icon_file != NULL)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
-			    GTK_SIGNAL_FUNC (toggle_custom_icon), 
-			    menu);
+	g_signal_connect (G_OBJECT (w), "toggled", 
+			  G_CALLBACK (toggle_custom_icon), 
+			  menu);
 	gtk_box_pack_start(GTK_BOX(box), w, TRUE, TRUE, 0);
 
 	w = gnome_icon_entry_new("icon", _("Browse"));
-	hack_icon_entry (GNOME_ICON_ENTRY (w));
 	menu->dialog_info->custom_icon_entry = w;
 	if (menu->custom_icon_file != NULL) {
-		hack_icon_entry_set_icon(GNOME_ICON_ENTRY(w),
-					 menu->custom_icon_file);
+		gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (w),
+					       menu->custom_icon_file);
 	}
 	if ( ! menu->custom_icon) {
 		gtk_widget_set_sensitive (w, FALSE);
 	}
 	gtk_box_pack_start(GTK_BOX(box), w, TRUE, TRUE, 0);
-	t = gnome_icon_entry_gtk_entry (GNOME_ICON_ENTRY (w));
-	gtk_signal_connect (GTK_OBJECT (t), "changed",
-			    GTK_SIGNAL_FUNC (textbox_changed),
-			    menu);
+	g_signal_connect (G_OBJECT (w), "changed",
+			  G_CALLBACK (textbox_changed),
+			  menu);
 
 	gtk_widget_grab_focus (global_main);
 
 	gtk_notebook_append_page (GTK_NOTEBOOK(notebook),
 				  vbox, gtk_label_new (_("Icon")));
 	
-	gtk_signal_connect (GTK_OBJECT(dialog), "destroy",
-			    GTK_SIGNAL_FUNC (properties_close_callback),
-			    menu);
+	g_signal_connect (G_OBJECT(dialog), "destroy",
+			  G_CALLBACK (properties_close_callback),
+			  menu);
 
-	gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
-			    GTK_SIGNAL_FUNC (dialog_clicked),
-			    menu);
+	g_signal_connect (G_OBJECT (dialog), "response",
+			  G_CALLBACK (dialog_response),
+			  menu);
+
+	/* Set the sensitivity of the frames as req'd. */
+	if (menu->main_menu) {
+		if (menu->global_main) {
+			gtk_widget_set_sensitive(menu->dialog_info->main_frame, FALSE);
+			gtk_widget_set_sensitive(menu->dialog_info->normal_frame, FALSE);
+		} else {
+			gtk_widget_set_sensitive(menu->dialog_info->main_frame, TRUE);
+			gtk_widget_set_sensitive(menu->dialog_info->normal_frame, FALSE);
+		}
+	} else {
+		gtk_widget_set_sensitive(menu->dialog_info->main_frame, FALSE);
+		gtk_widget_set_sensitive(menu->dialog_info->normal_frame, TRUE);
+	}
 
 	return dialog;
 }
@@ -671,8 +638,7 @@ menu_properties (Menu *menu)
 	g_return_if_fail (menu != NULL);
 
 	if (menu->prop_dialog != NULL) {
-		gtk_widget_show_now (menu->prop_dialog);
-		gdk_window_raise (menu->prop_dialog->window);
+		gtk_window_present (GTK_WINDOW (menu->prop_dialog));
 		return;
 	}
 
