@@ -7,17 +7,14 @@
 G_BEGIN_DECLS
 
 enum {
-	MAIN_MENU_USER = 1<<0,
-	MAIN_MENU_USER_SUB = 1<<1,
+	/* FIXME: should we move this array or keep the bitvalues
+	 * the same??? */
 	MAIN_MENU_SYSTEM = 1<<2,
 	MAIN_MENU_SYSTEM_SUB = 1<<3,
 	MAIN_MENU_DISTRIBUTION = 1<<4,
 	MAIN_MENU_DISTRIBUTION_SUB = 1<<5,
 	MAIN_MENU_KDE = 1<<6,
 	MAIN_MENU_KDE_SUB = 1<<7,
-	/* keep this for compatibility. */
-	MAIN_MENU_OBSOLETE_DEBIAN = 1<<8,
-	MAIN_MENU_OBSOLETE_DEBIAN_SUB = 1<<9,
 	MAIN_MENU_APPLETS = 1<<10,
 	MAIN_MENU_APPLETS_SUB = 1<<11,
 	MAIN_MENU_PANEL = 1<<12,
@@ -50,15 +47,17 @@ struct _Menu {
 	AppletInfo		*info;
 };
 
-void		load_menu_applet	(const char *path,
-					 gboolean main_menu,
-					 int main_menu_flags,
-					 gboolean global_main,
-					 gboolean custom_icon,
-					 const char *custom_icon_file,
+void		load_menu_applet	(const char  *path,
+					 gboolean     main_menu,
+					 int          main_menu_flags,
+					 gboolean     global_main,
+					 gboolean     custom_icon,
+					 const char  *custom_icon_file,
 					 PanelWidget *panel,
-					 int pos,
-					 gboolean exactpos);
+					 int          pos,
+					 gboolean     exactpos,
+					 const char  *gconf_key);
+
 void		add_menu_widget		(Menu *menu,
 					 PanelWidget *panel,
 					 GSList *menudirl,
@@ -74,8 +73,8 @@ void		make_panel_submenu	(GtkWidget *menu,
 					 gboolean fake_submenus,
 					 gboolean is_basep);
 
-GtkWidget *	create_panel_root_menu	(PanelWidget *panel,
-					 gboolean tearoff);
+GtkWidget *	create_panel_root_menu	(PanelWidget *panel);
+GtkWidget *     create_panel_context_menu (PanelWidget *panel);
 
 void		menu_properties		(Menu *menu);
 
@@ -85,28 +84,17 @@ void		panel_lock		(GtkWidget *widget,
 /*to be called on startup to load in some of the directories*/
 void		init_menus		(void);
 
-void		save_tornoff		(void);
-void		load_tornoff		(void);
-
 GtkWidget *	create_menu_at		(GtkWidget *menu,
 					 const char *menudir,
-					 gboolean applets,
 					 gboolean launcher_add,
-					 gboolean favourites_add,
 					 const char *dir_name,
 					 const char *pixmap_name,
 					 gboolean fake_submenus,
-					 gboolean force,
-					 gboolean title);
+					 gboolean force);
 GtkWidget *	create_fake_menu_at	(const char *menudir,
-					 gboolean applets,
 					 gboolean launcher_add,
-					 gboolean favourites_add,
 					 const char *dir_name,
-					 const char *pixmap_name,
-					 gboolean title);
-GtkWidget *	start_favourites_menu	(GtkWidget *menu,
-					 gboolean fake_submenus);
+					 const char *pixmap_name);
 
 void		submenu_to_display	(GtkWidget *menuw, gpointer data);
 gboolean	menu_need_reread	(GtkWidget *menuw);
@@ -116,14 +104,19 @@ void		setup_internal_applet_drag (GtkWidget *menuitem,
 GtkWidget *	create_root_menu	(GtkWidget *root_menu,
 					 gboolean fake_submenus,
 					 int flags,
-					 gboolean tearoff,
 					 gboolean is_basep,
-					 gboolean title);
+					 gboolean run_item);
 
 /* some gtk code cut-n-paste action */
 void		our_gtk_menu_position	(GtkMenu *menu);
 
-void		panel_add_favourite	(const char *source_dentry);
+void            menu_save_to_gconf   (Menu       *menu,
+				      const char *gconf_key);
+
+void            menu_load_from_gconf (PanelWidget *panel_widget,
+				      gint         position,
+				      const char  *gconf_key,
+				      gboolean     use_default);
 
 /* perhaps into basep-widget.h? */
 enum {
@@ -132,8 +125,16 @@ enum {
 	HIDEBUTTONS_NONE
 };
 
+#define PANEL_MENU_HAVE_ICONS_KEY "/desktop/gnome/menus/show-icons"
 gboolean panel_menu_have_icons   (void);
-gboolean panel_menu_have_tearoff (void);
+
+GtkWidget *	panel_menu_new		(void);
+void		panel_load_menu_image_deferred (GtkWidget *image_menu_item,
+						const char *image_filename,
+						const char *fallback_image_filename,
+						gboolean force_image);
+/* Note, bind the following on 'show' or some such */
+void		panel_make_sure_menu_within_screen (GtkMenu *menu);
 
 
 G_END_DECLS
