@@ -354,7 +354,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               panel_marshal_VOID__ENUM_POINTER_POINTER,
                               G_TYPE_NONE,
                               3,
-			      PANEL_TYPE_G_NOME__PANEL__BACK_TYPE,
+			      PANEL_TYPE_PANEL_BACK_TYPE,
 			      G_TYPE_POINTER,
 			      G_TYPE_POINTER);
 
@@ -1721,10 +1721,10 @@ panel_widget_set_id (PanelWidget *panel, const char *id)
 void
 panel_widget_set_new_id (PanelWidget *panel) 
 {
-	if (panel->unique_id != NULL) {
+	if (panel->unique_id)
 		g_free (panel->unique_id);
-	}
-	panel->unique_id = g_strdup (generate_unique_id ());
+
+	panel->unique_id = generate_unique_id ();
 }
 
 static void
@@ -1779,12 +1779,12 @@ panel_widget_new (gchar *panel_id,
         GTK_WIDGET_UNSET_FLAGS (panel, GTK_NO_WINDOW);
         GTK_WIDGET_SET_FLAGS (panel, GTK_CAN_FOCUS);
 
-	if (panel_id == NULL) {
-	panel->unique_id = g_strdup (generate_unique_id ());
-	}
+	if (!panel_id)
+		panel->unique_id = generate_unique_id ();
+
 	else {
-	  g_free (panel->unique_id);
-	  panel->unique_id = g_strdup (panel_id);
+		g_free (panel->unique_id);
+		panel->unique_id = g_strdup (panel_id);
 	} 
 
 	panel->back_type = back_type;
@@ -2874,8 +2874,6 @@ panel_widget_tab_move (PanelWidget *panel,
 	ad = panel->currently_dragged_applet;
 
 	if (!ad) {
-		GtkWidgetClass *class;
-
 		g_return_if_fail (GTK_WIDGET_HAS_FOCUS (panel));
 		
 		GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (panel), GTK_CAN_FOCUS);
@@ -2961,3 +2959,15 @@ void panel_widget_focus (PanelWidget *panel)
 	GTK_WIDGET_SET_FLAGS (panel, GTK_CAN_FOCUS);
       	gtk_widget_grab_focus (GTK_WIDGET (panel));
 }
+
+PanelOrient
+panel_widget_get_applet_orient (PanelWidget *panel)
+{
+	g_return_val_if_fail (PANEL_IS_WIDGET (panel), PANEL_ORIENT_UP);
+	g_return_val_if_fail (panel->panel_parent != NULL, PANEL_ORIENT_UP);
+
+	if (!BASEP_IS_WIDGET (panel->panel_parent))
+		return PANEL_ORIENT_DOWN;
+
+	return basep_widget_get_applet_orient (BASEP_WIDGET (panel->panel_parent));
+}
