@@ -124,6 +124,14 @@ struct _PanelWidgetClass
 			      GdkColor *color);
 	void (* applet_about_to_die) (PanelWidget *panel,
 				      GtkWidget *applet);
+	void (* push_move) (PanelWidget		*panel,
+                            GtkDirectionType	 dir);
+	void (* switch_move) (PanelWidget	*panel,
+                              GtkDirectionType	 dir);
+	void (* free_move) (PanelWidget		*panel,
+                            GtkDirectionType	 dir);
+	void (* tab_move) (PanelWidget	*panel,
+                           gboolean	 next);
 	void (* end_move) (PanelWidget	*panel);
 
 };
@@ -224,19 +232,11 @@ int		panel_widget_get_cursorloc	(PanelWidget *panel);
 /*needed for other panel types*/
 gboolean	panel_widget_is_cursor		(PanelWidget *panel,
 						 int overlap);
+/* set the focus on the panel */
+void            panel_widget_focus              (PanelWidget *panel);
 
+PanelOrient     panel_widget_get_applet_orient  (PanelWidget *panel);
 
-void            panel_widget_push_applet        (PanelWidget *panel,
-                                                 AppletData  *ad,
-                                                 GtkDirectionType dir);
-
-void            panel_widget_switch_applet      (PanelWidget *panel,
-                                                 AppletData  *ad,
-                                                 GtkDirectionType dir);
-
-void            panel_widget_free_move_applet   (PanelWidget *panel,
-                                                 AppletData  *ad,
-                                                 GtkDirectionType dir);
 extern gboolean panel_applet_in_drag;
 
 G_END_DECLS
